<?php

declare(strict_types=1);

namespace FiloBlu\RuleDayFromBirthday\Model\Rule\Condition\Customer;

use Exception;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Framework\Model\AbstractModel;
use Magento\Rule\Model\Condition\AbstractCondition;
use Magento\Rule\Model\Condition\Context;
use Magento\Sales\Model\ResourceModel\Order\CollectionFactory;
use Magento\Framework\Message\ManagerInterface;

/**
 * @class FirstOrder
 * @package FiloBlu\RuleDayFromBirthday\Model\Rule\Condition
 */
class FirstOrder extends AbstractCondition
{

    /**
     * @var CustomerRepositoryInterface
     */
    protected $customerRepository;
    /**
     * @var ManagerInterface
     */
    protected $manager;

    /**
     * @var CollectionFactory
     */
    protected $orderFactory;

    /**
     * FirstOrder constructor.
     * @param Context $context
     * @param CustomerRepositoryInterface $customerRepository
     * @param CollectionFactory $orderFactory
     * @param ManagerInterface $manager
     * @param array $data
     */
    public function __construct(
        Context $context,
        CustomerRepositoryInterface           $customerRepository,
        CollectionFactory                     $orderFactory,
        ManagerInterface                      $manager,
        array                                 $data = []
    ) {
        $this->customerRepository = $customerRepository;
        $this->orderFactory = $orderFactory;
        $this->manager = $manager;
        parent::__construct($context, $data);
    }

    /**
     * Load attribute options
     *
     * @return $this
     */
    public function loadAttributeOptions()
    {
        $attributes = [
            'customer_first_order' => __("Days from customer's first order")
        ];
        $this->setAttributeOption($attributes);

        return $this;
    }

    /**
     * Get input type
     * Possible values are: string, numeric, date, select, multiselect, grid, boolean
     *
     * @return string
     */
    public function getInputType()
    {
        return 'string';
    }

    /**
     * Get value element type
     *
     * We have the value element types as select, text
     * @return string
     */
    public function getValueElementType()
    {
        return 'text';
    }

    /**
     * @param AbstractModel $model
     * @return bool
     */
    public function validate(AbstractModel $model)
    {
        // Get the customer id of the current customer
        $customerId = (int)$model->getCustomerId();
        try {
            $order = $this->orderFactory->create($customerId);

            $all_orders = $order->getItems();
            $num_orders = count($all_orders);
            $order_created_at = [];


            if ($num_orders >= 1) {
                foreach ($all_orders as $items) {
                    $order_created_at [] = $items->getCreatedAt();
                }

                //retrieve created_at from first order only
                $val = $this->getDaysFromOrder($order_created_at[0]);
                $model->setData('customer_first_order', $val);

                return parent::validate($model);
            }

            return false;
        } catch (Exception $e) {
            $this->manager->addErrorMessage($e);
        }
    }

    /**
     * @param $order_created_at
     * @return int
     */
    protected function getDaysFromOrder($order_created_at)
    {
        $interval = strtotime("now") - strtotime($order_created_at);
        $num_of_days = (int)($interval / 86400);

        return $num_of_days;
    }
}
